{% extends "base.html" %}

{% block title %}
	&raquo; DC Contracts &raquo; Add
{% endblock %}

{% block nav %}
	<a href="/e/dc_contracts">DC Contracts</a> &raquo; Add
{% endblock %}

{% block content %}
	<form method="post">
		<fieldset>
			<legend>Add a DC contract</legend>
			<label>Participant</label>
			<select name="participant_id">
				{% for party in parties %}
					{{ input_option('participant_id', party.participant.id,
							party.participant.code + " : " + party.name) }}
				{% endfor %}
			</select>
			<label>Name</label> {{ input_text('name') }}
			<label>Start Date</label> {{ input_date('start', initial_date) }}
			<input type="submit" value="Add">
		</fieldset>
	</form>

{% endblock %}
