{% extends "base.html" %}

{% block title %}
	&raquo; Profile Classes
{% endblock %}

{% block nav %}
	Profile Classes
{% endblock %}

{% block content %}
	<table>
		<thead>
			<tr>
				<th>Code</th>
				<th>Description</th>
				<th>Valid From</th>
				<th>Valid To</th>
			</tr>
		</thead>
		<tbody>
			{% for pc in pcs %}
				<tr>
					<td><a href="/e/pcs/{{pc.id}}">{{pc.code}}</a></td>
					<td>{{pc.name}}</td>
					<td>{{pc.valid_from|hh_format}}</td>
					<td>{{pc.valid_to|hh_format}}</td>
				</tr>
			{% endfor %}
		</tbody>
	</table>
{% endblock %}
