{% extends "base.html" %}

{% block title %}
	&raquo; GSP Groups &raquo; {{gsp_group.code}}
{% endblock %}

{% block nav %}
	<a href="/e/gsp_groups">GSP Groups</a> &raquo; {{gsp_group.code}}
{% endblock %}

{% block content %}

	<table>
		<tr>
			<th>Code</th>
			<td>{{gsp_group.code}}</td>
		</tr>
		<tr>
				<th>Description</th>
				<td>{{gsp_group.description}}</td>
		</tr>
	</table>

{% endblock %}
